import { Signal, CustomSignalClass } from "./Signal";
/** Represents an object with observable members (use the constructor's argument to specify which members are observable and their initial values); may be extended into a derived class, but can be used as new Observable({...}) as well, OR use static .mixin(object) to mix into any object [requires ES5+ target] */
export declare class ObservableObject {
    /** Returns a new ObservableObject with properties copied from given object; (same as constructor but with typed return value) */
    static fromObject<T extends {}>(obj: T): T & ObservableObject;
    /** Mix in ObservableObject method and signal, and make all enumerable properties of given object observable */
    static mixin<T extends {}>(obj: T): T & ObservableObject;
    /** Create an object with (some) observable members, copying initial values from given object; `ObservableValue` instances are referenced directly (including getter and setter), Array properties are turned into `ObservableArray` instances, and plain Object instances into `ObservableObject` instances; also seals the new instance if the object parameter was sealed, and freezes it if the object parameter was frozen */
    constructor(obj?: {});
    /** Returns true if property with given name is observable */
    hasObservableProperty(name: string | number): boolean;
    /** Signal that is emitted when any property changes (but does not subscribe to any properties, i.e. only changes to properties already subscribed to, and properties with plain values [not getters] will trigger this signal) */
    PropertyChange: CustomSignalClass<string, typeof ObservableObject.PropertyChangeSignal>;
}
export declare namespace ObservableObject {
    /** Base class for signals that are emitted when any observed property is modified on an ObservableObject instance, with the name of the property as a parameter */
    class PropertyChangeSignal extends Signal<string> {
        /** The ObservableObject instance that this signal is used on */
        static target: ObservableObject;
    }
}
export declare namespace ObservableObject {
    /** Make property values observable and mix in `ObservableObject` signal and methods; second argument should be a hash of name-value pairs; `ObservableValue` instances are referenced directly (including getter and setter), Array properties are turned into `ObservableArray` instances, and plain Object instances into `ObservableObject` instances; returns the object itself [requires ES5+ target] */
    function makeObservable<T extends {}>(obj: T, spec: {}): T & ObservableObject;
    /** Make the property with given name observable [requires ES5+ target] */
    function makeObservable<T extends {}>(obj: T, member: string | number): T;
    /** Delete the property with given name and clear its observable value */
    function deleteObservableProperty(obj: {}, member: string | number): void;
    /** Returns true if property with given name is observable */
    function isObservableProperty(obj: {}, member: string | number): boolean;
}
/** *Property/accessor decorator*, makes a property observable on every instance [requires ES5+ target] [decorator] */
export declare function observable(target: Object, key: string, descriptor?: PropertyDescriptor): any;
/** *Property/accessor decorator*, makes a property observable on every instance and converts values to strings when read (empty string for null/undefined/NaN) [requires ES5+ target] [decorator] */
export declare function observable_string(target: Object, key: string, descriptor?: PropertyDescriptor): any;
/** *Property/accessor decorator*, makes a property observable on every instance and converts values to numbers using Number(...) when read [requires ES5+ target] [decorator] */
export declare function observable_number(target: Object, key: string, descriptor?: PropertyDescriptor): any;
/** *Property/accessor decorator*, makes a property observable on every instance and enforces that values are not undefined or null when read (throws TypeError) and that undefined/null values cannot be set (also throws TypeError) [requires ES5+ target] [decorator] */
export declare function observable_nonNull(target: Object, key: string, descriptor?: PropertyDescriptor): any;
/** *Property/accessor decorator*, makes a property observable on every instance and enforces that values are not undefined or null when read (throws TypeError) and that only instances of Object can be set, which are sealed automatically [requires ES5+ target] [decorator] */
export declare function observable_seal(target: Object, key: string, descriptor?: PropertyDescriptor): any;
/** *Property/accessor decorator*, makes a property observable on every instance and enforces that values are not undefined or null when read (throws TypeError) and that only instances of Object can be set, which are frozen automatically [requires ES5+ target] [decorator] */
export declare function observable_freeze(target: Object, key: string, descriptor?: PropertyDescriptor): any;
